## Example to run MIPGEM (doi: 10.1073/pnas.0907654107) on some sample
## data. Although the data used in this example builds up on a Sigma49
## experiment (doi: 10.1021/pr0604054), it is fictive in the sense that:
## - only part of the data is used to keep the example reasonably small
## - fictive gene models have been assigned to the proteins
## Please refer to the README file for more information about this code.
## Author: Sarah Gerster
## Date: 25 Nov 2011
## Last modified: 25 Nov 2011

## clean workspace
rm(list=ls())

## load R functions
source("fn_mipgem.R")


## ##################################################################### ##
## DATA (PREPROCESSING)                                                  ##
## ##################################################################### ##
## load data
load("sample_data.dat")

## check data format
checkInputData(peptides, proteins, gm, edgesPP, edgesGM)
peptides <- peptides[,c("pepID", "pepSEQ", "ppScore")]
nPeptides <- nrow(peptides)
proteins <- proteins[,c("protID", "accession")]
nProteins <- nrow(proteins)
edgesPP <- edgesPP[,c("pepID", "protID")]
gm <- gm[,c("gmID", "gmNAME")]
nGM <- nrow(gm)
edgesGM <- edgesGM[,c("protID", "gmID")]

## transform peptide scores
## -> logit of rescaled scores (to avoid logit(1))
peptides[,'ppScore'] <- scaledLogit(peptides[,'ppScore'])

## generate a graph with the peptides and the proteins
ppGraph <- buildBipartiteGraph(nPeptides, nProteins, edgesPP)
ppComponents <- connectedComp(ppGraph)

## generate the graph with peptides, proteins and gms
allGraph <- buildTripartiteGraph(nPeptides, nProteins, nGM, edgesPP,
                                 edgesGM)
allComponents <- connectedComp(allGraph)

## If any further data pre-processing is required (threshold peptides
## according to identification score, graph pruning, normalizations,...),
## it sould be done BEFORE running the MIPGEM block below.


## ##################################################################### ##
## MIPGEM                                                                ##
## ##################################################################### ##
## set constants for the piecewise linear functions (mixture model)
LEFT <- min(peptides[,"ppScore"])
RIGHT <- max(peptides[,"ppScore"])
MEDIAN <- median(peptides[,"ppScore"])
CONST1 <- ((RIGHT-MEDIAN)^2-(RIGHT-LEFT)^2)/(RIGHT-MEDIAN)^2
CONST2 <- 2/(RIGHT-MEDIAN)^2
CONST3 <- MEDIAN-LEFT

## estimate parameter value for the piecewise linear function (mixture
## model)
## -> usually one also has to optimize 'protPrior' (use a grid)
pPrior <- 0.5
## -> the optimization below typically takes quite some time
res <- optimize(f=nllPeptideDistr,lower=0,upper=0.1,tol=1e-3,
                ppGraph=ppGraph, nPeptides=nPeptides,
                protPrior=pPrior, verbose=FALSE)
optParam <- res$minimum

## compute the score (~presence probabilities) for all the proteins and,
## optionally, for all the gene models in the graph
## -> the computation below typically takes quite some time
res <- computeScores(proteins=proteins, gm=gm, param=optParam, ppGraph=ppGraph,
                     nPeptides=nPeptides, pPrior=pPrior)
proteins[,"protProb"] <-res[["protScore"]]
gm[,"gmProb"] <- res[["gmScore"]] 
