## MIPGEM (doi: 10.1073/pnas.0907654107)
## Author: Sarah Gerster
## Date: 25 Nov 2011
## Last modified: 25 Nov 2011

## ############### ##
## load R packages ##
## ############### ##
library(RBGL)
library(Rgraphviz)


## ######### ##
## CONSTANTS ##
## ######### ##
## if there are more than MAXPROTEINS proteins in a connected component,
## we use sampling instead of iteration over all possible protein
## configurations
MAXPROTEINS <- 10
## if we do sample, this is the number of samples to draw
NRSAMPLES <- 2**MAXPROTEINS


## ################ ##
## MIPGEM FUNCTIONS ##
## ################ ##
computeScores <- function(proteins, gm=NULL, param, ppGraph, nPeptides,
                          pPrior, verbose=FALSE)
{
  ## Purpose: estimate the proteins scores/concentrations with the
  ##          given parameter (estimates)
  ## ----------------------------------------------------------------------
  ## Arguments: proteins  : data frame with the proteins
  ##                        => needs to include the following column:
  ##                           - protID (unique identifier for each prot)
  ##            gm        : - set to NULL if only protein scores should 
  ##                          be computed
  ##                        - else: data frame with the gene models
  ##                       => needs to include the following columns:
  ##                          - gmID (unique identifier for each gm)
  ##            param     : parameter for the piecewise linear function
  ##                        in the mixture model
  ##            ppGraph   : peptide-protein bipartite graph
  ##            nPeptides : number of peptides in the bipartite graph
  ##            pPrior    : prior value for protein identification
  ##            verbose   : if TRUE, print some information to the screen
  ##                        for the user to follow the progress of the
  ##                        function
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 25 Nov 2011, 17:12
  res <- list()

  ## pre-computations needed for protein and gm scores
  A1 <- sapply(proteins[,"protID"], protProbA, protPres=1,
               param=optParam, ppGraph=ppGraph,
               nPeptides=nPeptides, protPrior=pPrior,
               verbose=verbose)
  A0 <- sapply(proteins[,"protID"], protProbA, protPres=0,
               param=optParam, ppGraph=ppGraph,
               nPeptides=nPeptides, protPrior=pPrior,
               verbose=verbose)

  ## protein scores
  res[["protScore"]] <- A1 / (A0 + A1)

  ## optionally also gene model scores
  if (!is.null(gm)) {
    res[["gmScore"]] <- sapply(gm[,"gmID"], geneModProb, param=param,
                               ppGraph=ppGraph, nPeptides=nPeptides,
                               A0=A0, A1=A1, protPrior= pPrior,
                               verbose=verbose)
  }
  
  return(res)
}


checkInputData <- function(peptides, proteins, gm, edgesPP, edgesGM)
{
  ## Purpose: check input data to make sure model will run properly
  ## ----------------------------------------------------------------------
  ## Arguments: peptides : dataframe holding the observed peptides
  ##                       => needs to include the following columns:
  ##                          - pepID: each peptide has a unique number,
  ##                            1 <= pepID <= nrow(peptides)       
  ##                          - pepSEQ: sequence of the peptide
  ##                          - ppScore: ProteinProphet (or similar) pep-
  ##                            tide identification score, 0 < ppScore <= 1
  ##            proteins : dataframe holding the proteins matching to at
  ##                       least one peptide in 'peptides'
  ##                       => needs to include the following columns:
  ##                          - protID: each protein has a unique number,
  ##                            (1+nrow(peptides)) <= protID <=
  ##                            (nrow(peptides) + nrwo(proteins))       
  ##                          - accession: protein name
  ##            gm       : dataframe holding the gene models matching to at
  ##                       least one protein in 'proteins'
  ##                       => needs to include the following columns:
  ##                          - gmID: each gene model has a unique number,
  ##                            (1+nrow(peptides)+nrow(proteins))
  ##                            <= gmID <=
  ##                            (nrow(peptides)+nrwo(proteins)+nrow(gm))  
  ##                          - gmNAME: gene model identifier
  ##            edgesPP  : dataframe with the edges of the bipartite
  ##                       peptide-protein graph. Each line corresponds to
  ##                       one edge
  ##                       => needs to include the following columns:
  ##                          - pepID
  ##                          - protID
  ##            edgesGM  : dataframe with the edges of the bipartite
  ##                       protein-gm graph. Each line corresponds to
  ##                       one edge
  ##                       => needs to include the following columns:
  ##                          - protID
  ##                          - gmID
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 25 Nov 2011, 14:44
  ## -> check 'peptides' dataframe
  stopifnot(all(c("pepID", "pepSEQ", "ppScore") %in% colnames(peptides)))
  if (ncol(peptides) > 3) 
    warning(paste("Only the following columns of 'peptides' will be used:",
                  "'pepID', 'pepSEQ' and 'ppScore'."))
  stopifnot(min(peptides[,"pepID"])==1 &&
            max(peptides[,"pepID"])==nrow(peptides) &&
            length(unique(peptides[,"pepID"]))==nrow(peptides))
  
  ## -> check 'proteins' dataframe
  stopifnot(all(c("protID", "accession") %in% colnames(proteins)))
  if (ncol(proteins) > 2) 
    warning(paste("Only the following columns of 'proteins' will be used:",
                  "'protID' and 'accession'."))
  stopifnot(min(proteins[,"protID"])==(nrow(peptides)+1) &&
            max(proteins[,"protID"])==(nrow(peptides)+nrow(proteins)) &&
            length(unique(proteins[,"protID"]))==nrow(proteins))
  stopifnot(all(peptides[,'ppScore'] > 0) &&
            all(peptides[,'ppScore'] <= 1))
  
  ## -> check 'edgesPP' dataframe
  stopifnot(all(c("pepID", "protID") %in% colnames(edgesPP)))
  if (ncol(edgesPP) > 2) 
    warning(paste("Only the following columns of 'edgesPP' will be used:",
                  "'pepID' and 'protID'."))
  stopifnot(all(edgesPP[,"pepID"] %in% peptides[,"pepID"]) &&
            all(edgesPP[,"protID"] %in% proteins[,"protID"]))
  
  ## -> check 'gm' dataframe
  stopifnot(all(c("gmID", "gmNAME") %in% colnames(gm)))
  if (ncol(gm) > 2) 
    warning(paste("Only the following columns of 'gm' will be used:",
                  "'gmID' and 'gmNAME'."))
  stopifnot(min(gm[,"gmID"])==(nrow(proteins)+nrow(peptides)+1) &&
            max(gm[,"gmID"])==(nrow(peptides)+nrow(proteins)+nrow(gm)) &&
            length(unique(gm[,"gmID"]))==nrow(gm))
  stopifnot(all(edgesGM[,"gmID"] %in% gm[,"gmID"]) &&
            all(edgesGM[,"protID"] %in% proteins[,"protID"]))
  
  ## -> check 'edgesGM' dataframe
  stopifnot(all(c("protID", "gmID") %in% colnames(edgesGM)))
  if (ncol(edgesGM) > 2) 
    warning(paste("Only the following columns of 'edgesGM' will be used:",
                  "'protID' and 'gmID'."))
}


nllPeptideDistr <- function(param, ppGraph, nPeptides, protPrior,
                            verbose=TRUE)
{
  ## Purpose: Return the negative log-likelihood of the probability
  ##          distribution of the peptide scores.
  ##          For the protein inference, we choose the parameter
  ##          setting minimizing this values.
  ## ----------------------------------------------------------------------
  ## Arguments: param:     parameter to be optimized (we are looking for
  ##                       the parameter yielding the lowest nll)
  ##                       -> slope b1 (slope of the first segment of the
  ##                       piecewise linear function)
  ##            ppGraph:   bipartite graph holding the peptide and protein
  ##                       sequences ('graphNEL' object)
  ##            nPeptides: number of peptides in the graph
  ##            protPrior: number between 0 and 1, prior probability of a
  ##                       protein beeing in the sample (assuming all
  ##                       proteins have the same 'prior'). 
  ##            verbose:   if TRUE print some infos to standard output
  ## Comments:  The function returns a scalar result.
  ##            Minimize nllPeptideDistr with otpimize to get the value
  ##            of the parameter param[1]. Optimize param[2] (protPrior)
  ##            on a grid to save computation time.
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 19 Mar 2008
  ## Last Modified: Sarah Gerster, Date: 29 Sep 2009

  if (!is.null(dim(param))) stop("check arguments of nllPeptideDistr")

  ## parameter vector
  param <- c(param, protPrior)

  ## condition on slopes: b1 > 0, b2 >= 0. This plus that fact that the 
  ## area below the piecewise linear function must be 1 leads to the 
  ## following condition:
  if ((2 < param[1]*(RIGHT-LEFT)^2) || (param[1]==0)){
    return(param[1]*(RIGHT-LEFT)^2*1e50) ## return a large number,
                                         ## proportional to b2
  }
  
  ## set a seed for the random number generator to ensure that the same
  ## samples are drawn for every set of parameters. Changing this seed \
  ## should not affect the results much.
  set.seed(17)

  ## get the probability densities if at least one matching protein is
  ## present and if none of the matching proteins is present in the
  ## sample
  probDensity <- matrix(nrow=nPeptides, ncol=3)
  colnames(probDensity) <- c("pepID", "Pres", "Abs")
  probDensity[,"pepID"] <- peptides[,"pepID"]
  probDensity[,"Pres"] <- sapply(peptides[,"pepID"],
                                 getPepDensityPrelim, proteinPres=1,
                                 param)
  probDensity[,"Abs"] <-
    rep(getPepDensityPrelim(peptideID=1, proteinPres=0, param),
        dim(probDensity)[1])
  
  ## Take all connected components with more than one element (=> holding
  ## at least one protein and one peptide)
  myComponents <- connectedComp(ppGraph)
  myComponents <- myComponents[sapply(myComponents,length)>1]
  
  out <- sum(log(sapply(myComponents, getComponentSummand, nPeptides,
                        ppGraph, param, probDensity, verbose)))
  if (verbose)
    cat('Negative log-likelihood = ', -out,'\n')
  return(-out)
}


getComponentSummand <- function(componentsList, nPeptides, ppGraph, param,
                                pepProbDensity, verbose = TRUE)
{
  ## Purpose: Compute the product of the peptide probability densities and
  ##          of the protein priors for a given state (combination of
  ##          protein presence) of a certain CC.
  ## ----------------------------------------------------------------------
  ## Arguments: componentsList: list of all the peptides and the proteins
  ##                            belonging to a certain connected component
  ##            nPeptides:      number of peptides in the whole graph
  ##            ppGraph:        bipartite Graph holding the peptide and
  ##                            the protein sequences ('graphNEL' object)
  ##            param:          Vector with two components: 
  ##                            param[1] = b1, slope of first segement of
  ##                            the piecewise linear function.
  ##                            param[2] = protPrior, 'prior' probability
  ##                            of the proteins being in the sample
  ##                            (assming all prot. have the same 'prior')
  ##            pepProbDensity: pre-computed desities
  ##            verbose:        if TRUE print some info to standard output
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 10 Apr 2008
  ## Last Modified: Sarah Gerster, Date: 29 Sep 2009

  ## list of proteins in the connected component
  ind.prot <- as.integer(componentsList) > nPeptides
  protArr <- componentsList[ind.prot]
  ## list of all the peptides in the connected component
  pepArr <- componentsList[!ind.prot]
  
  ## number of proteins in the CC
  nProt <- length(protArr)
  ## get all possible (or computationally feasable) configurations for
  ## the proteins and compute inner sum of equation S1
  if (nProt <= MAXPROTEINS) {
    ## if there are not more than MAXPROTEINS protein sequences in the CC,
    ## then use all the configurations
    combM <- sapply(1:nProt,get01Combination,nProt=nProt)
    ## compute the inner sum (in equation S1) for all possible combinations
    ## for the proteins states (present/absent in sample)
    inner.res <- sum(apply(combM, 1, getDensityPriorProd, pepArr, ppGraph,
                           protArr, param, pepProbDensity, verbose))
  } else {
    ## sample a 'computationally-reasonable' amount of configurations
    ## do this several times and take the average of the single runs
    inner.res <- 0
    
    ## draw NRSAMPLES times 2**MAXPROTEINS configurations
    for (ns in 1:NRSAMPLES) {
      ## sum over all runs
      ## vector of proteins present in the sample
      pres.prot <- protArr[as.logical(get01SamplingCombination(
                           protVec=protArr, protProb=param[2]))]
      ## vector of corresponding peptides
      pres.pep <- edgesPP[edgesPP[,"protID"] %in% pres.prot, "pepID"]
      abs.pep <- pepArr[!(pepArr %in% pres.pep)] 
      inner.res <- inner.res +
                   prod(pepProbDensity[pepProbDensity[,"pepID"] %in%
                                        pres.pep,"Pres"])*
                   prod(pepProbDensity[pepProbDensity[,"pepID"] %in%
                                        abs.pep,"Abs"])
    }
    ## average
    inner.res <- inner.res/NRSAMPLES
  }

  ## if something went wrong, write additional information to the
  ## standard output.
  #cat("inner.res = ", inner.res, "\n")
  if (inner.res <= 0) {
    cat("getComponentSummand result: ",inner.res,"\n")
    cat("componentsList: ",componentsList,"\n")
    cat("param: ",param,"\n")
  }
  
  return(inner.res)
}


getDensityPriorProd <- function(zv, pepArr, ppGraph, protArr, param,
                                pepProbDensity, verbose = TRUE)
{
  ## Purpose: Compute the product of the peptide probability densities and
  ##          the protein priors for a given state (protein presence) of a
  ##          certain CC.
  ## ----------------------------------------------------------------------
  ## Arguments: zv:             Protein presence: zv[i] = 0 means that
  ##                            protArr[i] is not in the sample, zv[i] = 1
  ##                            means that protArr[i] is in the sample
  ##            pepArr:         Peptides in the corresponding CC.
  ##            ppGraph:        Bipartite graph holding the peptide and
  ##                            the protein sequences ('graphNEL' object)
  ##            protArr:        Proteins in the corresponding CC
  ##            param:          Vector with two components: 
  ##                            param[1] = b1, slope of first segement of
  ##                            the piecewise linear function.
  ##                            param[2] = protPrior, 'prior' probability
  ##                            of the proteins being in the sample
  ##                            (assming all prot. have the same 'prior')
  ##            pepProbDensity: ........
  ##            verbose:        if TRUE print some infos to standard output
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 10 Apr 2008
  ## Last Modified: Sarah Gerster, Date: 29 Sep 2009
  ## compute product of peptide densities
  ## vector of proteins present in the sample
  pres.prot <- protArr[as.logical(zv)]
  ## vector of corresponding peptides
  pres.pep <- edgesPP[edgesPP[,"protID"] %in% pres.prot, "pepID"]
  abs.pep <- pepArr[!(pepArr %in% pres.pep)] 
  pepDensityProd <- prod(pepProbDensity[pepProbDensity[,"pepID"] %in%
                                        pres.pep,"Pres"])*
                    prod(pepProbDensity[pepProbDensity[,"pepID"] %in%
                                        abs.pep,"Abs"])
  ## compute product of protein 'priors' (assume same 'prior' for all
  ## proteins)
  protPriorProd <- param[2]^sum(zv) * (1-param[2])^sum(!zv)
  
  if (verbose) {
    cat("Protein IDs: ", protArr, "\n")
    cat("Protein presence: ", zv, "\n")
    cat("Product of the densities = ", pepDensityProd, "\n")
    cat("Product of the priors = ", protPriorProd, "\n")
  }
  
  res <- pepDensityProd * protPriorProd
  return(res)
}


getPepDensityPrelim <- function(peptideID, proteinPres, param)
{
  ## Purpose: Compute the probability density of the peptide score.
  ## ----------------------------------------------------------------------
  ## Arguments: peptideID:   Which peptide is of interest?
  ##            proteinPres: 1 if at least one protein matching to
  ##                         peptideID is in the sample, 0 otherwise
  ##            param:       parameter values for the model: protein prior
  ##                         and slope for mixture model (pwl function)
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 16 May 2008
  ## Last Modified: Sarah Gerster, Date: 29 Sep 2009

  t.score <- peptides[peptideID,"ppScore"]
  #cat("pep score = ", t.score, "\n")
  
  if (proteinPres) { ## at least one of the neighboring
                     ## proteins is in the sample
    ## piecewise linear approach
    if (t.score < MEDIAN) {
      ## first segment of the piecewise linear function
      out <- param[1]*(t.score-LEFT)
    } else {
      ## second segment of the piecewise linear function
      out <- (param[1]*CONST1+CONST2)*(t.score-MEDIAN)+param[1]*CONST3
    }
  } else { ## none of the proteins is in the sample
    ## uniform distribution
    out <- dunif(peptides[peptideID,"ppScore"], min=LEFT, max=RIGHT)
  }
  return(out)
}


getPepDensityDep <- function(peptideID, proteinPres, ppGraph, protArr, 
                          param)
{
  ## Purpose: Compute the probability density of the peptide score.
  ## ----------------------------------------------------------------------
  ## Arguments: peptideID:   Which peptide is of interest?
  ##            proteinPres: protein configuration vector. It holds 0
  ##                         (corresponding protein is not in the sample)
  ##                         or 1 (corresponding protein is in the sample).
  ##                         It holds the configuration of all proteins
  ##                         belonging to the same CC as 'peptideID'
  ##            ppGraph:     Bipartite graph holding the peptides and the
  ##                         proteins ('graphNEL' object)
  ##            protArr:     Proteins in the same CC as 'peptideID'
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 16 May 2008
  ## Last Modified: Sarah Gerster, Date: 10 Sep 2009
  
  ## find the neighboring proteins to peptideID
  neiProt <- as.integer(adj(ppGraph,as.character(peptideID))[[1]])
  ## get the state (present/absent) of these proteins
  neiZv <- proteinPres[protArr %in% neiProt]

  t.score <- peptides[peptideID,"ppScore"]
  
  if (any(as.logical(neiZv))) { ## at least one of the neighboring
                                ## proteins is in the sample
    ## piecewise linear approach
    if (t.score < MEDIAN) {
      ## first segment of the piecewise linear function
      out <- param[1]*(t.score-LEFT)
    } else {
      ## second segment of the piecewise linear function
      out <- (param[1]*CONST1+CONST2)*(t.score-MEDIAN)+param[1]*CONST3
    }
  } else { ## none of the proteins is in the sample
    ## uniform distribution
    out <- dunif(peptides[peptideID,"ppScore"], min=LEFT, max=RIGHT)
  }
  return(out)
}


get01Combination <- function(j, nProt) {
  ## Purpose: Function used to build a 'tableau de logique'. It is meant to
  ##          be used with apply on consecutive numbers from 1:nProt.
  ##          Example: sapply(1:3, get01Combination, 3) yields
  ##            0 0 0
  ##            0 0 1
  ##            0 1 0
  ##            0 1 1
  ##            1 0 0
  ##            1 0 1
  ##            1 1 0
  ##            1 1 1
  ##          where each column is generated by a call to get01Combination
  ## Comments: Only feasible for nProt and j < 22...
  ## ----------------------------------------------------------------------
  ## Arguments: j:     index of the column to generate
  ##            nProt: number of proteins (== number of colums in table to
  ##                   be generated trough usage of sapply; this table will
  ##                   have 2^nProt rows)
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 11 Apr 2008
  ## Last Modified: Sarah Gerster, Date: 10 Sep 2009
  
  return(rep(c(rep(0,2**(nProt-j)),rep(1,2**(nProt-j))),2**(j-1)))
}


get01SamplingCombination <- function(protVec, protProb) {
  ## Purpose: Generate one line of the combination matrix when using the
  ##          sampling. 
  ## ----------------------------------------------------------------------
  ## Arguments: protVec:  vector of proteins for which we need to sample.
  ##                      currently providing the length of the vector
  ##                      would actually be enough.
  ##            protProb: number between 0 and 1, prior probability of a
  ##                      protein beeing in the sample (assuming all
  ##                      proteins have the same 'prior').
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 11 Apr 2008
  ## Last Modified: Sarah Gerster, Date: 10 Sep 2009
  
  return(sample(c(0,1), size=length(protVec),
                prob=c((1-protProb), protProb), replace=TRUE))
}


protProbA <- function(proteinID, protPres, param, ppGraph, nPeptides,
                      protPrior=NULL, verbose=TRUE) {
  ## Purpose: Compute A(1) and A(0) in equation 9.
  ##          Return NA if the CC consists of only one protein (and no
  ##          peptides)
  ## ----------------------------------------------------------------------
  ## Arguments: proteinID: ID of protein of interest
  ##            protPres:  Is this protein in the sample (1) or not (0)?
  ##            param:     optimized parameter
  ##                       -> slope b1 (slope of the first segment of the
  ##                       piecewise linear function)
  ##            ppGraph:   bipartite graph holding the peptide and protein
  ##                       sequences ('graphNEL' object)
  ##            nPeptides: number of peptides in the graph
  ##            protPrior: number between 0 and 1, prior probability of a
  ##                       protein beeing in the sample (assuming all
  ##                       proteins have the same 'prior').
  ##            verbose:   if TRUE print some infos to standard output
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 26 Mar 2008
  ## Last Modified: Sarah Gerster, Date: 10 Sep 2009

  if (!is.null(dim(param))) stop("check arguments of nllPeptideDistr")

  ## parameter vector
  param <- c(param, protPrior)

  ## get the probability densities if at least one matching protein is
  ## present and if none of the matching proteins is present in the
  ## sample
  probDensity <- matrix(nrow=nPeptides, ncol=3)
  colnames(probDensity) <- c("pepID", "Pres", "Abs")
  probDensity[,"pepID"] <- peptides[,"pepID"]
  probDensity[,"Pres"] <- sapply(peptides[,"pepID"],
                                 getPepDensityPrelim, proteinPres=1,
                                 param)
  probDensity[,"Abs"] <-
    rep(getPepDensityPrelim(peptideID=1, proteinPres=0, param),
        dim(probDensity)[1])
  
  ## Take all connected components 
  myComponents <- connectedComp(ppGraph)

  ## To which connected component does the current protein belong?
  cc <- findCCIndex(proteinID, myComponents)
  
  ## TODO: get rid of that
  if (length(myComponents[[cc]]) == 1) return(NA)

  ## list of all proteins in the connected component
  ind.prot <- as.integer(myComponents[[cc]]) > nPeptides
  protArr <- myComponents[[cc]][ind.prot]
  
  ## list of all the peptides in the connected component
  pepArr <- myComponents[[cc]][!ind.prot]

  ## index of the current protein
  index.protID <- which(protArr == proteinID)
  
  ## number of proteins in the current CC
  nProt <- length(protArr)
  ## get all possible (or computationally feasable) configurations for
  ## the proteins in the current CC, except for the one with proteinID.
  ## The current protein (proteinID) is always present/absent depending
  ## on the value of the argument protPres.
  ## Compute the sum in equation 10.
  if (nProt <= MAXPROTEINS) {
    ## if there are not more than MAXPROTEINS+1 protein sequences in the
    ## CC, then use all the configurations
    combM <- matrix(protPres, nrow = 2**(nProt-1), ncol = nProt)
    ## TODO: no way to get rid of this if?
    if (nProt > 1) {
      combM[,-index.protID] <- sapply(1:(nProt-1), get01Combination,
                                      nProt=nProt-1)
    }
  
    ## compute the sum in equation 10 for all possible combinations
    ## for the proteins states (present/absent in sample).
    res <- sum(apply(combM, 1, getDensityPriorProd, pepArr, ppGraph,
                     protArr, param, pepProbDensity=probDensity, verbose))
  } else {
    ## sample a 'computationally-reasonable' amount of configurations
    ## do this several times and take the average of the single runs
    res <- 0
    
    ## draw NRSAMPLES times 2**MAXPROTEINS configurations
    for (ns in 1:NRSAMPLES) {
      ## set state of fixed protein (argument proteinID)
      vec1 <- rep(protPres, nProt)
      vec1[-index.protID] <- get01SamplingCombination(
                               protVec=protArr[-index.protID],
                               protProb=param[2])

      ## sum over all runs
      ## vector of proteins present in the sample
      pres.prot <- protArr[as.logical(vec1)]
      ## vector of corresponding peptides
      pres.pep <- edgesPP[edgesPP[,"protID"] %in% pres.prot, "pepID"]
      abs.pep <- pepArr[!(pepArr %in% pres.pep)] 
      res <- res +
             prod(probDensity[probDensity[,"pepID"] %in%
                                        pres.pep,"Pres"])*
             prod(probDensity[probDensity[,"pepID"] %in%
                                        abs.pep,"Abs"])*
             param[2]^protPres * (1-param[2])^(1-protPres)
    }
    ## average
    res <- res/NRSAMPLES
  }
  return(res)
}


geneModProb <- function(gmID, param, ppGraph, nPeptides, A0, A1,
                        protPrior, verbose=TRUE) {
  ## Purpose: compute the probability of a gene model being represented in
  ##          the sample by at least one protein as described in section 6
  ##          of the supporting information
  ## ----------------------------------------------------------------------
  ## Arguments: gmID:      ID of the gene model is of interest
  ##            param:     optimized parameter
  ##                       -> slope b1 (slope of the first segment of the
  ##                       piecewise linear function)
  ##            ppGraph:   bipartite graph holding the peptide and protein
  ##                       sequences ('graphNEL' object)
  ##            nPeptides: number of peptides in the graph
  ##            A0:        vector of values. Result from the computation of
  ##                       the help function defined in equation 10 with
  ##                       z=0. (vector of length nProteins)
  ##            A1:        vector of values. Result from the computation of
  ##                       the help function defined in equation 10 with
  ##                       z=1. (vector of length nProteins)
  ##            protPrior: number between 0 and 1, prior probability of a
  ##                       protein beeing in the sample (assuming all
  ##                       proteins have the same 'prior').
  ##            verbose:   if TRUE print some infos to standard output
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 16 Jul 2008
  ## Last Modified: Sarah Gerster, Date: 28 Dec 2009

  if (!is.null(dim(param))) stop("check arguments of nllPeptideDistr")

  ## parameter vector
  param <- c(param, protPrior)

### !!! USING GLOBAL VARIABLE !!! ###  
  ## get the probability densities if at least one matching protein is
  ## present and if none of the matching proteins is present in the
  ## sample
  probDensity <- matrix(nrow=nPeptides, ncol=3)
  colnames(probDensity) <- c("pepID", "Pres", "Abs")
  probDensity[,"pepID"] <- peptides[,"pepID"]
  probDensity[,"Pres"] <- sapply(peptides[,"pepID"],
                                 getPepDensityPrelim, proteinPres=1,
                                 param)
  probDensity[,"Abs"] <-
    rep(getPepDensityPrelim(peptideID=1, proteinPres=0, param),
        dim(probDensity)[1])
  
  ## get the proteins having common edges with with gmID
  proteinIDs <- edgesGM[which(edgesGM[,"gmID"]==gmID),"protID"]

  ## added 28 Dec 2009
  if (length(proteinIDs)==1) {
    return(A1[proteinIDs[1]-nPeptides]/(A0[proteinIDs[1]-nPeptides] +
                A1[proteinIDs[1]-nPeptides]))
  }
  
  ## Take all connected components
  myComponents <- connectedComp(ppGraph)
  
  ## To which component(s) does the GM (=> therefore any of the
  ## neighboring proteins) belong to?
  protComponents <- unique(sapply(proteinIDs, findCCIndex,
                                  componentsList=myComponents))

  ## initially assume GM is absent
  probGMAbsent <- 1
  ## for each peptide-protein CC having at least one edge to gmID
  for (cc in protComponents) {
    ## list and number of proteins in the connected component cc
    ind.prot <- as.integer(myComponents[[cc]]) > nPeptides
    protArr <- myComponents[[cc]][ind.prot]
    nProt <- length(protArr)
    
    ## list of proteins in the current CC belonging to the current GM
    myProteinIDs <- proteinIDs[proteinIDs %in% protArr]
    
    ## list of all the peptides in the current CC
    pepArr <- myComponents[[cc]][!ind.prot]
    
    ## assume all proteins of the GM are absent in the sample. The other
    ## proteins of the CC can be present or absent
    ## At the end: P(present) = 1- P(absent)
    
    ## proteins of the GM (myProteinIDs) (assumed to be absent)
    index.protIDs <- which(protArr %in% myProteinIDs)
    if ((nProt-length(myProteinIDs)) <= MAXPROTEINS) {
      ## if there are not more than MAXPROTEINS proteins with two possible
      ## states in the CC, then use all the possible protein
      ## configurations    
      combM <- matrix(0, nrow = 2**(nProt-length(myProteinIDs)),
                      ncol = nProt)
      ## TODO: no way to get rid of this if?
      if (nProt > length(myProteinIDs)) {
        combM[,-index.protIDs] <- sapply(1:(nProt-length(index.protIDs)),
                                         get01Combination,
                                         nProt=nProt-length(index.protIDs))
      }
      
      ## compute the sum (S6 in SI) for all possible combinations
      ## for the free protein states (present/absent) in sample.
      res <- sum(apply(combM, 1, getDensityPriorProd, pepArr, ppGraph,
                       protArr, param, pepProbDensity=probDensity,
                       verbose))
    } else {
      ## sample a 'computationally-reasonable' amount of configurations
      ## do this several times and take the average of the single runs     
      res <- 0
      
      ## draw NRSAMPLES times 2**MAXPROTEINS configurations
      for (ns in 1:NRSAMPLES) {
        ## set state of fixed protein (argument proteinID)
        vec1 <- rep(0,nProt)
        vec1[-index.protIDs] <- get01SamplingCombination(
                                     protVec=protArr[-index.protIDs],
                                                   protProb=param[2])
        ## sum over all samples
        ## vector of proteins present in the sample
        pres.prot <- protArr[vec1]
        ## vector of corresponding peptides
        pres.pep <- edgesPP[edgesPP[,"protID"] %in% pres.prot, "pepID"]
        abs.pep <- pepArr[!(pepArr %in% pres.pep)] 
        res <- res +
               prod(probDensity[probDensity[,"pepID"] %in%
                                        pres.pep,"Pres"])*
               prod(probDensity[probDensity[,"pepID"] %in%
                                        abs.pep,"Abs"])*
               param[2]^sum(vec1) * (1-param[2])^sum(!vec1)
      }
      
      ## average
      res <- res/NRSAMPLES
    }
    
    res <- res/(A0[myProteinIDs[1]-nPeptides] +
                A1[myProteinIDs[1]-nPeptides])

    ## product of computed probabilities if gm belongs to several peptide-
    ## protein CCs
    probGMAbsent <- probGMAbsent*res
  }

  ## P[gm present] = 1 - P[gm absent]
  gmProb <- 1-probGMAbsent
  return(gmProb)
}


findCCIndex <- function(ID, componentsList) {
  ## Purpose: Find the ID of the connected component in componentsList
  ##          holding the node (peptide, protein of gene model) ID.
  ## ----------------------------------------------------------------------
  ## Arguments: ID            : ID of the node (pep/prot/gm) of interest
  ##            componentsList: List holding the connected components (each
  ##                            entry in the list corresponds to one CC)
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 27 Jun 2008
  ## Last Modified: Sarah Gerster, Date: 11 Sep 2009
  
  for (cc in 1:length(componentsList)) {
    if (ID %in% componentsList[[cc]]) break
  }
  
  return(cc)
}


scaledLogit <- function(prob, scale=0.99)
{
  ## Purpose: compute scaled logit of 'prob'
  ## ----------------------------------------------------------------------
  ## Arguments: prob  : number in (0,1]
  ##            scale : factor to apply to 'prob' before computing the
  ##                    logit
  ## ----------------------------------------------------------------------
  ## Author: Sarah Gerster, Date: 25 Nov 2011
  return(log(scale*prob/(1-scale*prob)))
}


buildTripartiteGraph <- function(nPeptides, nProteins, nGM,
                                 edgesPP, edgesGM)
{
  ## generate a tri-partite graph with the peptides, the
  ## proteins and the gene models
  V <- as.character(1:(nPeptides+nProteins+nGM))
  edL <- vector("list", length=length(V))
  names(edL) <- V
  for (i in 1:nPeptides) {
    ind.i <- edgesPP[,"pepID"] == i
    edL[[i]] <- edgesPP[ind.i,"protID"]
  }
  for (j in (nPeptides+1):(nPeptides+nProteins)) {
    ind.pep <- edgesPP[,"protID"] == j
    ind.gm <- edgesGM[,"protID"] == j
    ed.new <- c(edgesGM[ind.gm,"gmID"], edgesPP[ind.pep,"pepID"])
    edL[[j]] <- ed.new
  }
  for (k in (nPeptides+nProteins+1):(nPeptides+nProteins+nGM)) {
    ind.k <- edgesGM[,"gmID"] == k
    edL[[k]] <- edgesGM[ind.k,"protID"]
  }
  return(new("graphNEL", nodes=V, edgeL=edL, edgemode="undirected"))
}


buildBipartiteGraph <- function(nPeptides, nProteins, edgesPP)
{
  ## generate a bipartite graph with the peptides and the
  ## proteins
  Vpp <- as.character(1:(nPeptides+nProteins))
  edLpp <- vector("list", length=length(Vpp))
  names(edLpp) <- Vpp
  for (i in 1:nPeptides) {
    ind.i <- edgesPP[,"pepID"] == i
    edLpp[[i]] <- edgesPP[ind.i,"protID"]
  }
  for (j in (nPeptides+1):(nPeptides+nProteins)) {
    ind.j <- edgesPP[,"protID"] == j
    edLpp[[j]] <- edgesPP[ind.j,"pepID"]
  }
  return(new("graphNEL", nodes=Vpp, edgeL=edLpp, edgemode="undirected"))
}
